"""Validation resolvers."""

from transportations_validator.validators.resolvers.condition import ConditionResolver
from transportations_validator.validators.resolvers.jurisdiction import JurisdictionResolver

__all__ = ["ConditionResolver", "JurisdictionResolver"]
